// Code generated by Herebyfile.mjs generate:enums from internal/checker/types.go. DO NOT EDIT.
export var TypePredicateKind;
(function (TypePredicateKind) {
    TypePredicateKind[TypePredicateKind["This"] = 0] = "This";
    TypePredicateKind[TypePredicateKind["Identifier"] = 1] = "Identifier";
    TypePredicateKind[TypePredicateKind["AssertsThis"] = 2] = "AssertsThis";
    TypePredicateKind[TypePredicateKind["AssertsIdentifier"] = 3] = "AssertsIdentifier";
})(TypePredicateKind || (TypePredicateKind = {}));
//# sourceMappingURL=typePredicateKind.js.map