// Code generated by Herebyfile.mjs generate:enums from internal/core/compileroptions.go. DO NOT EDIT.
export var NewLineKind;
(function (NewLineKind) {
    NewLineKind[NewLineKind["None"] = 0] = "None";
    NewLineKind[NewLineKind["CRLF"] = 1] = "CRLF";
    NewLineKind[NewLineKind["LF"] = 2] = "LF";
})(NewLineKind || (NewLineKind = {}));
//# sourceMappingURL=newLineKind.enum.js.map