export var CommentDirectiveType;
(function (CommentDirectiveType) {
    CommentDirectiveType[CommentDirectiveType["ExpectError"] = 0] = "ExpectError";
    CommentDirectiveType[CommentDirectiveType["Ignore"] = 1] = "Ignore";
})(CommentDirectiveType || (CommentDirectiveType = {}));
//# sourceMappingURL=commentDirectiveType.js.map