export var ScriptTarget;
(function (ScriptTarget) {
    ScriptTarget[ScriptTarget["ES2015"] = 2] = "ES2015";
    ScriptTarget[ScriptTarget["ES2016"] = 3] = "ES2016";
    ScriptTarget[ScriptTarget["ES2017"] = 4] = "ES2017";
    ScriptTarget[ScriptTarget["ES2018"] = 5] = "ES2018";
    ScriptTarget[ScriptTarget["ES2019"] = 6] = "ES2019";
    ScriptTarget[ScriptTarget["ES2020"] = 7] = "ES2020";
    ScriptTarget[ScriptTarget["ES2021"] = 8] = "ES2021";
    ScriptTarget[ScriptTarget["ES2022"] = 9] = "ES2022";
    ScriptTarget[ScriptTarget["ES2023"] = 10] = "ES2023";
    ScriptTarget[ScriptTarget["ES2024"] = 11] = "ES2024";
    ScriptTarget[ScriptTarget["ES2025"] = 12] = "ES2025";
    ScriptTarget[ScriptTarget["ESNext"] = 99] = "ESNext";
    ScriptTarget[ScriptTarget["JSON"] = 100] = "JSON";
    ScriptTarget[ScriptTarget["Latest"] = 99] = "Latest";
})(ScriptTarget || (ScriptTarget = {}));
//# sourceMappingURL=scriptTarget.js.map